#include <stdio.h>
#include "keymap.h"
#include "keyboard.h"

static const int ADDR_PINS[] = {2, 3, 4, 5};
static const int DATA_PINS[] = {A0, A1, A2, A3, A4};
static const int LED_PIN = 13;
static const int ANALOG_THRESHOLD = 128;

bool keyStates[NUM_KEYS];

void setup() {
  // Address-Pins
  for(int i=0; i<4; i++) {
    pinMode(ADDR_PINS[i], OUTPUT);
    digitalWrite(ADDR_PINS[i], LOW);
  }

  // LED
  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, LOW);

  // Data-Pins
  for(int i=0; i<5; i++) {
    pinMode(DATA_PINS[i], INPUT);
  }

  memset(keyStates, 0, sizeof(keyStates));
  
  initKeyboard();
  Serial.begin(115200);
}

void writeAddress(int addr) {
  digitalWrite(ADDR_PINS[0], addr & 0x1 ? HIGH : LOW);
  digitalWrite(ADDR_PINS[1], addr & 0x2 ? HIGH : LOW);
  digitalWrite(ADDR_PINS[2], addr & 0x4 ? HIGH : LOW);
  digitalWrite(ADDR_PINS[3], addr & 0x8 ? HIGH : LOW);
}

void readKeyStates(bool* keyStates) {  
  for(int addr = 0; addr < 16; addr++) {
    writeAddress(addr);
    delay(1);

    for(int line = 0; line < 4; line++) {
      int keyIndex = line * 16 + addr;
      if(keyIndex < NUM_KEYS) {
        keyStates[keyIndex++] = (analogRead(DATA_PINS[line]) < ANALOG_THRESHOLD);
      }
    }
  }

  // Last key (enter) is inverted
  keyStates[NUM_KEYS-1] = !keyStates[NUM_KEYS-1];
}

void printKeyStates(bool* keyStates) {
  char str[NUM_KEYS+1];
  for(int i = 0; i < NUM_KEYS; i++) {
    str[i] = keyStates[i] ? '1' : '0';
  }
  
  str[NUM_KEYS] = '\0';
  Serial.println(str);
  Serial.println("");
}

void detectKeyPress(bool* oldStates, bool* newStates) {
  bool keyPressed = false;
  
  char str[10];
  for(int i = 0; i < NUM_KEYS; i++) {
    if(newStates[i]) {
      keyPressed = true;
      
      if(! oldStates[i]) {
        int code = 0;
        if(newStates[META]) {
          code = KEY_MAP_META[i];
        } else if(newStates[LEFT_SHIFT] || newStates[RIGHT_SHIFT]) {
          code = KEY_MAP_UPPER[i];
        } else {
          code = KEY_MAP_LOWER[i];
        }
  
        sendKey(code & ~(SHIFT_FLAG | ALTGR_FLAG), code & SHIFT_FLAG, code & ALTGR_FLAG);
      }
    }
  }

  // Set LED
  digitalWrite(LED_PIN, keyPressed ? HIGH : LOW);
}

void loop() {
  bool oldStates[NUM_KEYS];
  memcpy(oldStates, keyStates, sizeof(keyStates));
  
  readKeyStates(keyStates);
  detectKeyPress(oldStates, keyStates);
}
